pub mod broadcast;
pub mod error;
pub mod listen;

use std::net::SocketAddr;

use anyhow::Result;
use axum::{
  extract::Path, http::StatusCode, response::Html, routing::get, Router,
};
use tera::{Context, Tera};
use tokio::runtime::Runtime;

use crate::server::{broadcast::broadcast, listen::listen};

lazy_static::lazy_static! {
  pub static ref TERA: Tera = {
    let mut tera = Tera::default();
    tera.add_raw_template("index.html", include_str!("../static/index.html")).unwrap();
    tera
  };
}

#[derive(Debug, Parser)]
pub struct ServerOpts {
  /// Address `host:port' to bind to
  #[clap(long = "bind-addr")]
  bind_addr: Option<String>,
}

pub fn server_main(opts: ServerOpts) -> Result<()> {
  let runtime = Runtime::new()?;

  runtime.block_on(server_async_main(opts))
}

async fn server_async_main(opts: ServerOpts) -> Result<()> {
  println!("Hellosu {opts:?}");

  // build our application with a single route
  let app = Router::new()
    .route("/watch/:room_id", get(watch_page))
    .route("/ws/:room_id", get(listen))
    .route("/broadcast", get(broadcast));

  // run it with hyper on localhost:8200
  let addr: SocketAddr = opts
    .bind_addr
    .unwrap_or_else(|| String::from("127.0.0.1:8200"))
    .parse()?;
  println!("Listening on {addr:?}...");
  axum::Server::bind(&addr)
    .serve(app.into_make_service())
    .await?;

  Ok(())
}

async fn watch_page(
  Path((room_id,)): Path<(String,)>,
) -> Result<Html<String>, StatusCode> {
  let mut ctx = Context::new();
  ctx.insert("room_id", &room_id);

  let html = TERA
    .render("index.html", &ctx)
    .map_err(|_| StatusCode::INTERNAL_SERVER_ERROR)?;
  Ok(Html(html))
}
